package Bundle::Xmms;

$VERSION = '1.00';

1;

__END__

=head1 NAME

Bundle::Xmms - Bundle for xmms remote control shell

=head1 SYNOPSIS

C<perl -MCPAN -e 'install Bundle::Xmms'>

=head1 CONTENTS

Term::ReadKey

Term::ReadLine::Perl

Term::ANSIColor

MPEG::MP3Info

Xmms

=head1 DESCRIPTION

The modules installed by this bundle will give you full power of the
xmms remote control shell.

=head1 AUTHOR

Doug MacEachern
